function out=inject_attack(t, sampleFreq, MVs, attackParam)
global Ts_base;
global I_PULSE I_SCALING I_RAMP I_RANDOM I_REPLAY;
global replayBuffer replayPos lastValBuffer;

%% Unpack parameters
aType = attackParam(1);
aStart = attackParam(2);
aStop = attackParam(3);
aTarget = attackParam(4);

%% Initialize output - no changes.
out=MVs;

%% In case we do not need to run attack yet, simply return.
if t < aStart || t > aStop
    if aType == I_REPLAY
        %% In the case of the Replay attack we need to store in the buffer
        %% ahead
        aStartRec = attackParam(5); % The start of the recording period
        aEndRec = attackParam(6); % The end of the recording period
        if t >= aStartRec && t <= aEndRec
            if exist('replayBuffer') > 0
                replayBuffer = [replayBuffer MVs(aTarget)];
            else
                replayBuffer = [MVs(aTarget)];
            end
        end
    end
    
    lastValBuffer = MVs;
    return;
end

switch aType
    case I_PULSE
        %% Grab the attack value. We will issue aVal/2 at the middle
        %% and aVal in the attack interval.
        aVal = lastValBuffer(aTarget);%attackParam(5);
            
        %% Compute the middle of the interval - where the pulse is applied.
        t_middleSt = (aStop-aStart)/2.0 - (aStop-aStart)/4 + aStart;
        t_middleEnd = (aStop-aStart)/2.0 + (aStop-aStart)/4 + aStart;
            
        %% Apply the attack.
        if t >= t_middleSt && t <= t_middleEnd
            out(aTarget) = aVal/attackParam(5);
        else
            out(aTarget) = aVal;
        end
        
    case I_SCALING
        %% Grab the attack value. This is the scale of the attack.
        aVal = attackParam(5);
        out(aTarget) = aVal * out(aTarget);
    
    case I_RAMP
        %% Grab the ramp value. This is the scale (in percentage) of the ramp attack.
        aVal = attackParam(5) * lastValBuffer(aTarget);
        out(aTarget) = out(aTarget) + aVal * (t-aStart);
    
    case I_RANDOM
        %% Grab the interval for generating the random number.
        percVal = attackParam(5) * lastValBuffer(aTarget);
        out(aTarget) = lastValBuffer(aTarget) + percVal.*(2*(rand(1,1)-0.5));
        
    case I_REPLAY
        %% Replay the recorded data
        if t == aStart
            % Make sure we reset
            replayPos = 1;
        end
        [n m] = size(replayBuffer);
        
        if replayPos >= m
            replayPos = 1;
        end
        out(aTarget) = replayBuffer(replayPos);
        replayPos = replayPos + 1;
    otherwise
end