function [rabcl normRabcl C CNorm]=caia(y, yNoInterv, tStart, tEnd, sTime)
%% Calculation of impact for each intervention
% rabcl - calculated impact
% normrabcl - normalized impact in [0,1]
% C - C intermediate matrix with cross covariances (see in the article)
% y - 3D matrix.

%% Grab the number of interventions (iCount), the number of observed
%% variables (jCount), and the number of samples (tCount).
[iCount jCount sCount] = size(y);


%% Init structures
rabcl=[];
C=[];

%% Transform time to sample index
tStart = tStart / sTime + 1;
tEnd = tEnd / sTime + 1;

for i = 1:iCount
    r = [];
    tEndTmp = tEnd;
    %% Verify if the process was shut-down. If yes, we only compute CAIA
    %% for the normal functioning interval.
    posNaN = find(isnan(squeeze(y(i, 1, :))))';
    if size(posNaN,2) >= 1 && posNaN(1) > 0
        fprintf('Found NaN starting with: %d.\n\t Computing CAIA with reduced interval!', posNaN(1));
        tEndTmp = posNaN(1)-1;
        continue;
    end
    
    for j = 1:jCount
        % Grab the intervention-free measurements for j
        a = yNoInterv(j, tStart:tEndTmp);
        
        % Grab the measurements for intervention i
        b = squeeze(y(i, j, tStart:tEndTmp));
        
        % Compute mean values for both two cases above
        am = mean(a);
        bm = mean(b);
        
        % Compute the cross-covariance
        %r = [r (abs(a-am)*abs(b-bm))/(std(a)*std(b)*size(a,2))];
        r = [r std(b)/std(a)];
    end
    
    % Compute the mean cross-covariance and build the 'C' matrix
    rabcl(i) = mean(r);
    if mean(r) ~=0
        C = [C;r];
    end
end

%% Compute the impact and normalized impact values
rabcl=rabcl./sum(rabcl);
normRabcl=(rabcl - min(rabcl)) ./ ( max(rabcl) - min(rabcl) );

%% Compute the min-max normalization
C(isnan(C)) = max(max(C));
CNorm = C;%(max(max(C))-C)/(max(max(C)));

