%% Clear everything
%clear all;
%clear sim;

%% Setup path
addpath('./_core_model');

%% Load parameters for the VAM
run('_core_model/VAMSymbolicNames.m'); % (code written to access symbols specific to TEP)

%% Load parameters related to attacks
run('./attack_types.m');

%% Create attack parameters that will be forwarded to the TEP model
global attackParam simout u_history simDuration atStart atEnd;

%% Build attack parameters (interventions) and launch simulations
simDuration = 1000; %Minutes
sampleFreq = 0.1; % Per minute
sampleCount = simDuration / sampleFreq + 1;
%y_nointerv = zeros(CTRLVAR_COUNT + 1, OBSVAR_COUNT, sampleCount);

% No intervention
%fprintf('Running simulation without an intervention...\n\n');
%attackParam = attackBuilder(NOATTACK, 1, 1.1, 0);
%test_VAcPlant(simDuration, 0, sampleFreq);

%y_nointerv = simout;


%% Now compute the CAIA on a wide range of attacks
atStart = 900; atEnd = 903;

targets = [1:CTRLVAR_COUNT];
y = run_full_attack(I_PULSE, atStart, atEnd, simDuration, sampleCount, targets, 2);
[rabcl normRabcl Craw C1] = caia(y, y_nointerv, atStart, atEnd, sampleFreq);
I1=find(rabcl > 0);
I1= [I1' ones(length(I1),1)];
y = run_full_attack(I_PULSE, atStart, atEnd, simDuration, sampleCount, targets, 5);
[rabcl normRabcl Craw C2] = caia(y, y_nointerv, atStart, atEnd, sampleFreq);
I2=find(rabcl > 0);
I2= [I2' 2*ones(length(I2),1)];
targets = [1:16 18:19 21:CTRLVAR_COUNT];
y = run_full_attack(I_SCALING, atStart, atEnd, simDuration, sampleCount, targets, 1.1);
[rabcl normRabcl Craw C3] = caia(y, y_nointerv, atStart, atEnd, sampleFreq);
I3=find(rabcl > 0);
I3= [I3' 3*ones(length(I3),1)];
targets = [1:16 18:19 21:CTRLVAR_COUNT];
y = run_full_attack(I_SCALING, atStart, atEnd, simDuration, sampleCount, targets, 1.2);
[rabcl normRabcl Craw C4] = caia(y, y_nointerv, atStart, atEnd, sampleFreq);
I4=find(rabcl > 0);
I4= [I4' 4*ones(length(I4),1)];
targets = [1:CTRLVAR_COUNT];
y = run_full_attack(I_RAMP, atStart, atEnd, simDuration, sampleCount, targets, 0.01);
[rabcl normRabcl Craw C5] = caia(y, y_nointerv, atStart, atEnd, sampleFreq);
I5=find(rabcl > 0);
I5= [I5' 5*ones(length(I5),1)];
targets = [1:25];
y = run_full_attack(I_RAMP, atStart, atEnd, simDuration, sampleCount, targets, 0.05);
[rabcl normRabcl Craw C6] = caia(y, y_nointerv, atStart, atEnd, sampleFreq);
I6=find(rabcl > 0);
I6= [I6' 6*ones(length(I6),1)];
targets = [1:CTRLVAR_COUNT];
y = run_full_attack(I_RANDOM, atStart, atEnd, simDuration, sampleCount, targets, 0.2);
[rabcl normRabcl Craw C7] = caia(y, y_nointerv, atStart, atEnd, sampleFreq);
I7=find(rabcl > 0);
I7= [I7' 7*ones(length(I7),1)];
y = run_full_attack(I_RANDOM, atStart, atEnd, simDuration, sampleCount, targets, 0.5);
[rabcl normRabcl Craw C8] = caia(y, y_nointerv, atStart, atEnd, sampleFreq);
I8=find(rabcl > 0);
I8= [I8' 8*ones(length(I8),1)];
targets = [1:25];
y = run_full_attack(I_REPLAY, atStart, atEnd, simDuration, sampleCount, targets, [1 2]);
[rabcl normRabcl Craw C9] = caia(y, y_nointerv, atStart, atEnd, sampleFreq);
I9=find(rabcl > 0);
I9= [I9' 9*ones(length(I9),1)];
targets = [1:25];
y = run_full_attack(I_REPLAY, atStart, atEnd, simDuration, sampleCount, targets, [2 4]);
[rabcl normRabcl Craw C10] = caia(y, y_nointerv, atStart, atEnd, sampleFreq);
I10=find(rabcl > 0);
I10= [I10' 10*ones(length(I10),1)];
INTERV = [I1' I2' I3' I4' I5' I6' I7' I8' I9' I10']; 
C = [C1' C2' C3' C4' C5' C6' C7' C8' C9' C10'];

%% Plot the output
% subplot(2,2,1);
% plot(simout(:, OUT_REACTOR_PRESSURE));
% subplot(2,2,2);
% plot(simout(:, OUT_REACTOR_LEVEL));
% subplot(2,2,3);
% plot(simout(:, OUT_REACTOR_TEMP));
% subplot(2,2,4);
% plot(simout(:, OUT_PURGE_RATE));
